# Copyright 2016 Jochen Fabricius
# Distribution and modification is encouraged

# returns a A/B/C/D/ITU468-weighted frequency response of $fr
proc weighting {fr type} {
  set FS [esweep::samplerate -obj $fr]
  set dirac [esweep::create -type complex -samplerate $FS -size [esweep::size -obj $fr]]
  esweep::generate dirac -obj dirac
  switch -- $type {
    A {
      set filter [esweep::createFilterFromCoeff -type highpass -samplerate $FS \
        -Fp 20.6 -Qp 0.5]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type highpass -samplerate $FS -Fp 12200 -Qp 0.5]]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type integrator -samplerate $FS -Fp 107.7 -Qp 0.5]]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type integrator -samplerate $FS -Fp 737.9 -Qp 0.5]]
      set sc 3.890e3
    }
    B {
      set filter [esweep::createFilterFromCoeff -type bandpass -samplerate $FS \
        -Fp 20.6 -Qp 0.5]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type highpass -samplerate $FS -Fp 12200 -Qp 0.5]]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type integrator -samplerate $FS -Fp 158.5 -Qp 0.5]]
      set sc 7.674e4
    }
    C {
      set filter [esweep::createFilterFromCoeff -type highpass -samplerate $FS \
        -Fp 20.6 -Qp 0.5]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type lowpass -samplerate $FS -Fp 12200 -Qp 0.5]]
      set sc 1
    }
    D {
      set filter [esweep::createFilterFromCoeff -type linkwitz -samplerate $FS \
        -Fp 3136.5 -Qp 0.92 -Fz 1018.8 -Qz 0.9]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type integrator -samplerate $FS -Fp 282.7 -Qp 0.5]]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type integrator -samplerate $FS -Fp 1160 -Qp 0.5]]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type differentiator -samplerate $FS -Fp 20e3 -Qp 0.5]]
      set sc 2.6e3
    }
    ITU468 {
      set f 8000
      set q 1
      set filter [esweep::createFilterFromCoeff -type lowpass -samplerate $FS \
        -Fp $f -Qp $q]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type lowpass -samplerate $FS -Fp $f -Qp $q]]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type integrator -samplerate $FS -Fp $f]]
      set filter [esweep::appendFilter -filter $filter -append \
        [esweep::createFilterFromCoeff -type differentiator -samplerate $FS -Fp 20e3]]
      set sc 56.2
    }
    default {
      # nothing
      return $fr
    }
  }
  esweep::filter -signal dirac -filter $filter
  esweep::expr dirac * $sc
  esweep::fft -inplace -obj dirac
  if {[esweep::type -obj $fr] eq {polar}} {
    esweep::toPolar -obj dirac
  }
  esweep::expr fr * $dirac
  return $fr
}

