# $from and $to are in dBA, n is the number of steps, limit is the maximum allowed output voltage
macro limit {type vinit vlimit dlimit} {
  switch -glob -- $type {
    tw* {
      :set Signal,Definition,Bandwidth {3000 20000}
    }
    mid* {
      :set Signal,Definition,Bandwidth {300 3000}
    }
    w* {
      :set Signal,Definition,Bandwidth {20 300}
    }
    lo* {
      :set Signal,Definition,Bandwidth {20 3000}
    }
    default {
      :set Signal,Definition,Bandwidth {20 20000}
    }
  }
  :set Signal,Definition,Bins 10/2
  :set Signal,Definition,Spacing log

  :set Measure,Mode dual

  set colors [list red purple darkblue darkgreen brown black yellow] 

  # set initial output level
  :set Output,Level $vinit
  # increase output level by 3 dB each step
  set step 1.414

  # measure outside of the loop to get a fixed signal definition
  set ID [:measure default]
  set sigdef [:query $ID SigDef]

  set i 0
  while {1} {
    :copy $ID
    set ID [:paste]
    # wait for the display to update
    # this serves also as a pause to reduce heating of VC
    after 500
    :eval $ID -weighting A
    set rms [:query $ID RMSweighted]
    set td [expr {100*pow(10, ([:query $ID TD]-[:query $ID RMSweighted])/20)}]
    :modify $ID Name [format "%.1f dBA - %.2f%% TD @ %.1f Vrms" $rms $td [:query $ID OutLvl]] Color [lindex $colors [expr {$i % [llength $colors]}]]

    if {$td > $dlimit} {
      :message {Maximum distortion exceeded}
      break
    }
    # check the next voltage step
    if {[expr {$step*[:query $ID OutLvl]}] >= $vlimit} {
      :message {Maximum allowed output voltage reached}
      break
    }
    :set Output,Level [expr {$step*[:set Output,Level]}]
    if {[catch {set ID [:measure default]} msg]} {
      :message $msg
      break
    }
    incr i
  }
  return -code continue
}
